
import requests
from .google.geolocation import Geolocation
from .yelp.business import BusinessSearch


class Client:
    def __init__(self):
        self.session = requests.session()
        self.yelp = BusinessSearch(self)
        self.geolocation = Geolocation(self)




import geocoder
import uuid
import os

from urllib.parse import quote


# API constants, you shouldn't have to change these.
API_HOST = 'https://www.googleapis.com'
SEARCH_PATH = '/geolocation/v1/geolocate'


class Geolocation:
    def __init__(self, client):
        self.client = client

    def geolocate(self):
        try:
            API_KEY = os.environ['GOOGLE_API_KEY']
            # print('google api key...')
            url_params = {
                'key': API_KEY,
            }

            body = {'macAddress': ':'.join(['{:02x}'.format(
                (uuid.getnode() >> ele) & 0xff) for ele in
                range(0, 8 * 6, 8)][::-1]), }

            url_params = url_params or {}
            url = '{0}{1}'.format(API_HOST, quote(SEARCH_PATH.encode('utf8')))

            response = self.client.session.request(
                'POST', url, params=url_params, data=body)
            return response['location']['lat'], response['location']['lng']

        except KeyError:
            # dont use this irl. just look up your lat long at this point.
            # useful to test without using an google geolocation API key tho
            # print('no google api key, try geocoder...')
            g = geocoder.ip('me')
            return g.latlng[0], g.latlng[1]





class ClickData:
    def __init__(
            self,
            term,
            location,
            latitude,
            longitude,
            radius,
            categories,
            locale,
            limit,
            sort_by,
            price,
            attributes,
            verbose):
        self.term = term.replace(' ', '+')
        self.location = location.replace(' ', '+') if location else None
        self.latitude = latitude
        self.longitude = longitude
        self.term = term
        self.radius = radius
        self.categories = categories
        self.locale = locale
        self.limit = limit
        self.sort_by = sort_by
        self.price = price
        self.attributes = attributes
        self.verbose = verbose

    def set_location(self, lat, ln):

        self.latitude = lat
        self.longitude = ln





'''
{'id': 'zLViW6kDKNLIYsGkBCNorg',
'alias': 'paesano-ristorante-italiano-san-jose',
'name': 'Paesano Ristorante Italiano',
'image_url': 'https://s3-media4.fl.yelpcdn.com/bphoto/0K_JiyyYtQDw1gw8xY2YWQ/o.jpg',
'is_closed': False,
'url': 'https://www.yelp.com/biz/paesano-ristorante-italiano-san-jose?adjust_creative=Tb7mKIyLJPzAUf1fWI9E9g&utm_campaign=yelp_api_v3&utm_medium=api_v3_business_search&utm_source=Tb7mKIyLJPzAUf1fWI9E9g',
'review_count': 1019,
'categories': [{'alias': 'italian', 'title': 'Italian'}],
'rating': 4.0, 'coordinates': {'latitude': 37.336166540678, 'longitude': -121.89867135144},
'transactions': ['delivery'],
'price': '$$',
'location': {'address1': '350 W Julian St', 'address2': 'Ste 1', 'address3': '', 'city': 'San Jose', 'zip_code': '95110', 'country': 'US', 'state': 'CA', 'display_address': ['350 W Julian St', 'Ste 1', 'San Jose, CA 95110']},
'phone': '+14082179327',
'display_phone': '(408) 217-9327',
'distance': 484.37802455489407}
'''


class BusinessData:
    def __init__(self, data):
        self.id = data['id']
        self.alias = data['alias']
        self.name = data['name']
        self.image_url = data['image_url']
        self.is_closed = data['is_closed']
        self.url = data['url']
        self.display_url = data['url'].split('?')[0]
        self.review_count = data['review_count']
        self.categories = data['categories']
        self.transactions = data['transactions']
        self.price = data['price']
        self.location = data['location']
        self.display_location = " ".join(data['location']['display_address'])
        self.phone = data['phone']
        self.display_phone = data['display_phone']
        self.rating = data['rating']
        self.distance = data['distance']

        self.convert_price()
        self.convert_categories()

    def __repr__(self):
        return self.name

    def __str__(self):
        return " ".join([self.id, self.name])

    def extend_details(self, data):
        self.hours = data['hours']
        self.photos = data['photos']
        self.coordinates = data['coordinates']
        # yelp why is this an array???
        self.is_open_now = data['hours'][0]['is_open_now']

    def get_full_printable(self):
        return [
            self.name,
            "",
            '{} 🌟 {} reviews {}'.format(self.rating, self.review_count, self.display_price),
            "",
            '📞 {}'.format(self.display_phone),
            '🌐 {}'.format(self.display_url),
            '📫 {}'.format(self.display_location),
            '',
            "{} {}".format(", ".join([x['title'] for x in self.categories]), self.display_categories),
            "",
            'Open Now' if self.is_open_now else 'Closed',
            # "",         
            # "{}".format(self.reviews[0]['text'])
        ]

    def get_printable(self):
        return [
            self.name,
            '{}🌟({}) {} {}'.format(self.rating, self.review_count,
                                   self.display_price,
                                   self.display_categories),
            self.display_location,
            self.display_url,
            ""
        ]

    def convert_price(self):
        self.display_price = '💲' * len(self.price)

    def convert_categories(self):
        ret = ""
        category_to_emoji = {
            'seafood': '🦞',
            'burgers': '🍔',
            'tacos': '🌮',
            'foodtrucks': '🚛',
            'italian': '🍝',
            'german': '',
            'bars': '🍻',
            'beergardens': '🍻',
            'mexican': '🌮',
            'venues': '',
            'fishnchips': '🦞',
            'newmexican': '🌮',
            'greek': '🥙',
            'mediterranean': '🥙',
            'cajun': '🦐',
            'chicken_wings': '🍗',
            'chickenshop': '🍗',
            'tapasmallplates': '🍢',
            'tapas': '🍢',
            'wine_bars': '🍷',
            'whiskybars': '🥃',
            'shanghainese': '🍜',
            'cantonese': '🥮',
            'chinese': '🥮',
            'coffee': '☕',
            'hkcafe': '🥮',
            'bbq': '🥩',
            'southern': '🥩',
            'cambodian': '',
            'persian': '',
            'japanese': '🍣',
            'hotpot': '🍲',
            'diyfood': '',
            'noodles': '🍜',
            'vegan': '',
            'panasian': '',
            'asianfusion': '',
            'lounges': '',
            'soup': '🍲',
            'ramen': '🍜',
            'japacurry': '🍛',
            'brewpubs': '🍺',
            'hawaiian': '🌴',
            'tikibars': '🌴',
            'brasseries': '',
            'musicvenues': '',
            'food_court': '',
            'carribean': '',
            'tradamerican': '',
            'pizza': '🍕',
            'hotdog': '🌭',
            'hotdogs': '🌭',
            'pubs': '🍺',
            'gastropubs': '🍺',
            'french': '',
            'kebab': '🍢',
            'halal': '',
            'asianfusion': '',
            'vietnamese': '🍜',
            'cocktailbars': '',
            'korean': '🍚',
            'poke': '',
            'steak': '🥩',
            'bubbletea': '🍹',
            'himalayan': '',
            'newamerican': '',
            'breakfast_brunch': '🥓',
            'salads': '🥗',
            'thai': '',
        }

        for c in self.categories:
            if c['alias'] in category_to_emoji:
                ret += category_to_emoji[c['alias']]

        self.display_categories = ret



import os
from urllib.parse import quote

API_KEY = os.environ['YELP_API_KEY']
API_HOST = 'https://api.yelp.com'
SEARCH_PATH = '/v3/businesses/search'
BUSINESS_PATH = '/v3/businesses/'  # Business ID will come after slash.
REVIEWS_PATH = '/reviews'
SEARCH_LIMIT = 20


class BusinessSearch:
    def __init__(self, client):
        self.client = client

    def request(self, host, path, api_key, url_params=None):
        url_params = url_params or {}
        url = f"{host}{quote(path.encode('utf8'))}"
        headers = {
            'Authorization': 'Bearer %s' % api_key,
        }

        # print(u'Querying {0} ...'.format(url))

        response = self.client.session.request(
            'GET', url, headers=headers, params=url_params)

        return response.json()

    def get_business(self, business_id):
        """Query the Business API by a business ID.

        Args:
            business_id (str): The ID of the business to query.

        Returns:
            dict: The JSON response from the request.
        """
        business_path = BUSINESS_PATH + business_id

        return self.request(API_HOST, business_path, API_KEY)

    def get_business_reviews(self, business_id):
        """Query the Business API by a business ID.

        Args:
            business_id (str): The ID of the business to query.

        Returns:
            dict: The JSON response from the request.
        """
        business_path = BUSINESS_PATH + business_id + REVIEWS_PATH

        return self.request(API_HOST, business_path, API_KEY)

    def query_business_search_api(self, url_params):
        """Queries the API by the input values from the user.
        Args:
            term (str): The search term to query.
            location (str): The location of the business to query.
        """
        response = self.request(API_HOST, SEARCH_PATH,
                                API_KEY, url_params=url_params.__dict__)

        # print(response)

        return response.get('businesses')



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="welp",
    version="0.0.4.1",
    author="Alex Ye",
    author_email="alexzye1@gmail.com",
    description="what should i eat?",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/azye/welp",
    entry_points={'console_scripts': ['welp=welp.__main__:welp']},
    packages=setuptools.find_packages(),
    python_requires='>=3.6',
    install_requires=[
        'click',
        'geocoder',
    ],
)



import click
from .welp import Welp
from .click_data import ClickData
from .business_data import BusinessData
from .api.client import Client


@click.group()
def welp():
    pass


@click.command()
@click.option('--term', default='Restaurants', type=click.STRING)
@click.option('--location', type=click.STRING)
@click.option('--latitude', type=click.FLOAT)
@click.option('--longitude', type=click.FLOAT)
@click.option('--radius', default=5000, type=click.INT)
@click.option('--categories', default="", type=click.STRING)
@click.option('--locale', default="en_US", type=click.STRING)
@click.option('--limit', default=50, type=click.INT)
@click.option('--sort-by', default='best_match', type=click.STRING)
@click.option('--price', default='1,2,3', type=click.STRING)
@click.option('--attributes', default='', type=click.STRING)
@click.option('--verbose', default=False, type=click.BOOL)
def search(term, location, latitude, longitude, radius,
           categories, locale, limit, sort_by, price, attributes, verbose):
    welp = Welp()
    client = Client()

    click_data = ClickData(
        term,
        location,
        latitude,
        longitude,
        radius,
        categories,
        locale,
        limit,
        sort_by,
        price,
        attributes,
        verbose)

    if not click_data.latitude and not click_data.longitude \
       and not click_data.location:
        lat, ln = client.geolocation.geolocate()
        click_data.set_location(lat, ln)

    bus = client.yelp.query_business_search_api(click_data)
    # print(welp.api_client.yelp.get_business('zLViW6kDKNLIYsGkBCNorg'))

    data = []
    for b in bus:
        # print(b)
        data.append(BusinessData(b))

    welp.ui.set_data(data)
    welp.ui.open_curses_ui()


welp.add_command(search)



import curses
import math
import webbrowser
from .api.client import Client

rows_in_entry = 5


class SelectionWindow:
    def __init__(self, data, stdscr, window):
        client = Client()
        self.window_box = window
        self.stdscr = stdscr
        self.rows, self.cols = stdscr.getmaxyx()
        extendedData = client.yelp.get_business(data.id)
        data.extend_details(extendedData)
        data.reviews = client.yelp.get_business_reviews(data.id)['reviews']
        self.business = data
        # should query for review data and extended business data here

    def print_business(self):
        NORMAL_TEXT = curses.A_NORMAL

        if not self.business:
            self.window_box.addstr(1, 1, "Not a valid business 🍴",
                                   NORMAL_TEXT)
        else:
            business_data = self.business.get_full_printable()

            for i in range(self.rows):
                if i == len(business_data):
                    break

                self.window_box.addstr(i, 1, business_data[i], NORMAL_TEXT)

        self.stdscr.refresh()
        self.window_box.refresh()

    def poll_draw_render(self):
        key_press = self.stdscr.getch()

        # run until quit keys are pressed
        while key_press != 27 and key_press != 113:
            # refresh the rows for printing
            self.rows, self.cols = self.stdscr.getmaxyx()

            if key_press == curses.KEY_DOWN or key_press == 106:
                pass

            if key_press == curses.KEY_UP or key_press == 107:
                pass

            if key_press == curses.KEY_LEFT or key_press == 104:
                pass

            if key_press == curses.KEY_RIGHT or key_press == 108:
                pass

            if key_press == 10:
                webbrowser.open_new_tab(self.business.display_url)

            self.window_box.erase()
            self.print_business()

            key_press = self.stdscr.getch()


class CursesWindow:        

    def set_data(self, data):
        self.data = data

    def print_selections(self):
        HIGHLIGHT_TEXT = curses.color_pair(1)
        NORMAL_TEXT = curses.A_NORMAL

        if len(self.data) == 0:
            self.window_box.addstr(1, 1, "No results found 🍴",
                                   NORMAL_TEXT)
        else:
            # if self.rows % rows_in_entry != 0:
            #     self.rows -= (self.rows % rows_in_entry)
            max_entries = int(self.rows / rows_in_entry)

            rows_remaining = self.rows - (rows_in_entry * max_entries)
            
            if rows_remaining >= 4:
                max_entries += 1

            data_start_idx = int(self.current_page * max_entries)
            i = 0

            for entry in self.data[data_start_idx:data_start_idx +
                                   max_entries]:
                for row in entry.get_printable():
                    if i < self.rows:
                        if self.position * rows_in_entry <= i < self.position * \
                            rows_in_entry + 4:
                            self.window_box.addstr(i, 0, ' ', HIGHLIGHT_TEXT)

                        self.window_box.addstr(i, 1, row, NORMAL_TEXT)
                        i += 1

        self.stdscr.refresh()
        self.window_box.refresh()

    def open_curses_ui(self):
        curses.wrapper(self.print_window)

    def init_curses(self, stdscr):
        # initializes a color pair 1 where black is the foreground (text) and
        # cyan is the background
        curses.init_pair(1, curses.COLOR_BLACK, curses.COLOR_CYAN)
        # disable terminal cursor
        curses.curs_set(0)

        self.stdscr = stdscr
        self.rows, self.cols = stdscr.getmaxyx()
        self.position = 0
        self.current_page = 0
        self.window_box = curses.newwin(self.rows, self.cols)

    def poll_draw_render(self):
        # HIGHLIGHT_TEXT = curses.color_pair(1)
        # NORMAL_TEXT = curses.A_NORMAL
        key_press = self.stdscr.getch()
        max_valid_rows = self.rows - (self.rows % rows_in_entry)
        max_entries = int(self.rows / rows_in_entry)
        rows_remaining = self.rows - (rows_in_entry * max_entries)

        if rows_remaining >= 4:
            max_entries += 1

        self.max_pages = int(math.ceil(len(self.data) / max_entries) - 1)
        # run until quit keys are pressed
        while key_press != 27 and key_press != 113:
            # refresh the rows for printing
            self.rows, self.cols = self.stdscr.getmaxyx()

            if key_press == curses.KEY_RESIZE:
                self.position = 0
                self.current_page = 0
                self.rows, self.cols = self.stdscr.getmaxyx()
                max_valid_rows = self.rows - (self.rows % rows_in_entry)
                self.max_pages = math.ceil(len(self.data) / max_entries) - 1

            if key_press == 10:
                data_start_idx = int(self.current_page * max_entries)

                current_entry = self.data[int(data_start_idx):int(data_start_idx +
                                          max_entries)][int(self.position)]
                # print(current_entry)
                self.window_box.erase()
                
                w = SelectionWindow(current_entry, self.stdscr, self.window_box)
                w.print_business()
                w.poll_draw_render()

            if key_press == curses.KEY_DOWN or key_press == 106:
                curr_data_idx = (self.current_page * max_entries) + self.position

                if curr_data_idx + 1 < len(self.data):
                    if self.position * rows_in_entry < max_valid_rows - rows_in_entry:
                        self.position += 1
                    else:
                        self.current_page += 1
                        self.position = 0
                    
            if key_press == curses.KEY_UP or key_press == 107:
                self.position -= 1

                if self.position < 0:
                    if self.current_page > 0:
                        self.current_page -= 1
                        self.position = (max_valid_rows / rows_in_entry) - 1
                    else:
                        self.position = 0

            if key_press == curses.KEY_LEFT or key_press == 104:
                if self.current_page > 0:
                    self.current_page -= 1
                    self.position = 0

            if key_press == curses.KEY_RIGHT or key_press == 108:
                if self.current_page < self.max_pages:
                    self.current_page += 1
                    self.position = 0

            self.window_box.erase()
            self.print_selections()

            key_press = self.stdscr.getch()

    def print_window(self, stdscr):
        self.init_curses(stdscr)
        self.print_selections()
        self.poll_draw_render()
